// -----------------------------------------------------------------------
// Litmus - DKFZ SIDT unit testing library
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostic and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See litmusCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/litmus/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 4433 $ (last changed revision)
// @date    $Date: 2013-01-22 14:38:20 +0100 (Di, 22 Jan 2013) $ (last change date)
// @author  $Author: floca $ (last changed by)
*/


#ifndef __LIT_VALUE_RECORDER_TPP
#define __LIT_VALUE_RECORDER_TPP

#include <algorithm>

namespace lit 
{
    template <typename TRecordValue>
    const typename ValueRecorder<TRecordValue>::RecordValueVectorType&
      ValueRecorder<TRecordValue>::
      getRecordValues() const
    {
      return _recordedValues;
    };

    template <typename TRecordValue>
    typename ValueRecorder<TRecordValue>::RecordCountType
      ValueRecorder<TRecordValue>::
      getTotalRecordValueCount() const
    {
      return _recordedValues.size();
    };

    template <typename TRecordValue>
    typename ValueRecorder<TRecordValue>::RecordIndexVectorType
      ValueRecorder<TRecordValue>::
      getRecordValueSequence(const RecordValueType& value) const
    {
      RecordIndexVectorType findings;

      typename RecordValueVectorType::const_iterator location = std::find(_recordedValues.begin(),_recordedValues.end(),value);

      while (location != _recordedValues.end())
      {
        findings.push_back(location-_recordedValues.begin());
        location = std::find(++location,_recordedValues.end(),value);
      }

      return findings;
    };

    template <typename TRecordValue>
    typename ValueRecorder<TRecordValue>::RecordCountType
      ValueRecorder<TRecordValue>::
      getRecordValueCount(const RecordValueType& value) const
    {
      return getRecordValueSequence(value).size();
    };

    template <typename TRecordValue>
    bool
      ValueRecorder<TRecordValue>::
      checkRecordValue(const RecordValueType& value) const
    {
      typename RecordValueVectorType::const_iterator location = std::find(_recordedValues.begin(),_recordedValues.end(),value);

      return location!= _recordedValues.end();
    };

    template <typename TRecordValue>
    bool
      ValueRecorder<TRecordValue>::
      checkRecordValueAt(const RecordValueType& value, const RecordIndexType& index) const
    {
      bool result = false;

      if (index<_recordedValues.size())
      { //index is inside the vector, thus a recorded value exists
        result = _recordedValues[index] == value;
      }

      return result;
    };

    template <typename TRecordValue>
    void
      ValueRecorder<TRecordValue>::
      resetRecorder()
    {
      _recordedValues.clear();
    };

    template <typename TRecordValue>
    void
      ValueRecorder<TRecordValue>::
      recordValue(RecordValueType value)
    {
      _recordedValues.push_back(value);
    };
}

#endif
